*!*    CLEAR

*!*    s = CREATEOBJECT("_SHA1")
*!*    ?"SHA1 Self Test is "+IIF(s.TestSHA1(),"O.K.","ERROR")
*!*    ?S.SHA1("1234")
*!*    ?

*************************


***********************

DEFINE CLASS _SHA1 AS CUSTOM

	FUNCTION TestSHA1
	RETURN (THIS.SHA1("abc");
		==CHR(0xA9)+CHR(0x99)+CHR(0x3E)+CHR(0x36)+CHR(0x47)+CHR(0x06)+CHR(0x81)+CHR(0x6A)+CHR(0xBA)+CHR(0x3E)+CHR(0x25)+CHR(0x71)+CHR(0x78)+CHR(0x50)+CHR(0xC2)+CHR(0x6C)+CHR(0x9C)+CHR(0xD0)+CHR(0xD8)+CHR(0x9D)) ;
		AND (THIS.SHA1("abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq");
		==CHR(0x84)+CHR(0x98)+CHR(0x3E)+CHR(0x44)+CHR(0x1C)+CHR(0x3B)+CHR(0xD2)+CHR(0x6E)+CHR(0xBA)+CHR(0xAE)+CHR(0x4A)+CHR(0xA1)+CHR(0xF9)+CHR(0x51)+CHR(0x29)+CHR(0xE5)+CHR(0xE5)+CHR(0x46)+CHR(0x70)+CHR(0xF1))

	FUNCTION SHA1 (pcStringIn)
	LOCAL lcIn, i, j, l, T, ;
		H0, H1, H2, H3, H4, ;
		A, B, C, D, E
	LOCAL ARRAY W(80)

	l = MOD(LEN(pcStringIn)+1+8,64)
* 1. padding
	lcIn = pcStringIn + ;
		CHR(0x80) + ;
		REPLICATE(CHR(0x00), 64-MOD(LEN(pcStringIn)+1+8,64))+;
		THIS.NumToBin (8*LEN(pcStringIn), 8)

	H0 = 0x67452301
	H1 = 0xEFCDAB89
	H2 = 0x98BADCFE
	H3 = 0x10325476
	H4 = 0xC3D2E1F0

	FOR i = 1 TO LEN(lcIn)-64+1 STEP 64
		FOR T=0 TO 15
			W(T+1) = 256*256*256*ASC(SUBSTR(lcIn,i+4*T+0,1))+;
				256*256*ASC(SUBSTR(lcIn,i+4*T+1,1))+;
				256*ASC(SUBSTR(lcIn,i+4*T+2,1))+;
				ASC(SUBSTR(lcIn,i+4*T+3,1))
		ENDFOR

		FOR T = 16 TO 79
			W(T+1) = THIS.Sn(1,;
				BITXOR(BITXOR(BITXOR(W(T+1-3),W(T+1-8)),W(T+1-14)),W(T+1-16)))
		ENDFOR
		A = H0
		B = H1
		C = H2
		D = H3
		E = H4
		FOR T = 0 TO 79
			TEMP = THIS.Sn(5,A) + THIS.F(T,B,C,D) + E + W(T+1) + THIS.K(T)
			E = D
			D = C
			C = THIS.Sn(30,B)
			B = A
			A = TEMP

		ENDFOR
		H0 = H0 + A
		H1 = H1 + B
		H2 = H2 + C
		H3 = H3 + D
		H4 = H4 + E
	ENDFOR
	RETURN ;
		THIS.NumToBin(H0,4)+;
		THIS.NumToBin(H1,4)+;
		THIS.NumToBin(H2,4)+;
		THIS.NumToBin(H3,4)+;
		THIS.NumToBin(H4,4)

	HIDDEN FUNCTION Sn(N, X)
		RETURN BITOR(BITLSHIFT(X,N), BITRSHIFT(X,32-N))

	HIDDEN FUNCTION F(T,B,C,D)
		ASSERT BETW(T,0,79)
		DO CASE
		CASE BETW(T,  0, 19)
			RETURN BITOR(BITAND(B,C),BITAND(BITNOT(B),D))
		CASE BETW(T, 20, 39)
			RETURN BITXOR(BITXOR(B,C),D)
		CASE BETW(T, 40, 59)
			RETURN BITOR(BITOR(BITAND(B,C),BITAND(B,D)),BITAND(C,D))
		CASE BETW(T, 60, 79)
			RETURN BITXOR(BITXOR(B,C),D)
		ENDCASE

	HIDDEN FUNCTION K(T)
		ASSERT BETW(T,0,79)
		DO CASE
		CASE BETW(T,  0, 19)
			RETURN 0x5A827999
		CASE BETW(T, 20, 39)
			RETURN 0x6ED9EBA1
		CASE BETW(T, 40, 59)
			RETURN 0x8F1BBCDC
		CASE BETW(T, 60, 79)
			RETURN 0xCA62C1D6
		ENDCASE

	HIDDEN FUNCTION NumToBin (pnNumber, pnNoOfBytes)
		LOCAL Num, i, ret
		ret = ""
		Num = pnNumber
		IF Num<0
* 0xFFFFFFF          -1
* 0x8000000 -2147483648
			Num = Num+0xFFFFFFFF+1
		ENDIF
		FOR i = 1 TO pnNoOfBytes
			ret = CHR(MOD(Num,256))+ret
			Num = INT(Num/256)
		ENDFOR
		RETURN ret

ENDDEFINE

